@@ -210,7 +210,6 @@ void
 find_script_hash (systemtap_session& s, const string& script)
 {
   stap_hash h(get_base_hash(s));
-  struct stat st;
 
   // Hash getuid.  This really shouldn't be necessary (since who you
   // are doesn't change the generated output), but the hash gets used
@@ -223,27 +222,13 @@ find_script_hash (systemtap_session& s, const string& script)
   h.add("Bulk Mode (-b): ", s.bulk_mode);
   h.add("Timing (-t): ", s.timing);
   h.add("Prologue Searching (-P): ", s.prologue_searching);
-  h.add("Ignore Vmlinux (--ignore-vmlinux): ", s.ignore_vmlinux);
-  h.add("Ignore Dwarf (--ignore-dwarf): ", s.ignore_dwarf);
-  h.add("Consult Symtab (--kelf, --kmap): ", s.consult_symtab);
   h.add("Skip Badvars (--skip-badvars): ", s.skip_badvars);
   h.add("Privilege (--privilege): ", s.privilege);
   h.add("Compatible (--compatible): ", s.compatible);
   h.add("Omit Werror (undocumented): ", s.omit_werror);
   h.add("Prologue Searching (-P): ", s.prologue_searching);
   h.add("Error suppression (--suppress-handler-errors): ", s.suppress_handler_errors);
-  if (!s.kernel_symtab_path.empty())	// --kmap
-    {
-      h.add("Kernel Symtab Path: ", s.kernel_symtab_path);
-      if (stat(s.kernel_symtab_path.c_str(), &st) == 0)
-        {
-	  // NB: stat of /proc/kallsyms always returns size=0, mtime=now...
-	  // which is a good reason to use the default /boot/System.map-2.6.xx
-	  // instead.
-          h.add("Kernel Symtab Size: ", st.st_size);
-	  h.add("Kernel Symtab Timestamp: ", st.st_mtime);
-        }
-    }
+  h.add("Suppress Time Limits (--suppress-time-limits): ", s.suppress_time_limits);
   for (unsigned i = 0; i < s.c_macros.size(); i++)
     h.add("Macros: ", s.c_macros[i]);
 
