@@ -1,5 +1,5 @@
 /* main header file for Linux
- * Copyright (C) 2005-2013 Red Hat Inc.
+ * Copyright (C) 2005-2014 Red Hat Inc.
  * Copyright (C) 2005-2006 Intel Corporation.
  *
  * This file is part of systemtap, and is free software.  You can
@@ -81,13 +81,27 @@ static void _stp_exit(void);
 #define stap_hlist_for_each_entry_safe(a,b,c,d,e) (void) b; hlist_for_each_entry_safe(a,c,d,e)
 #endif
 
+#ifndef preempt_enable_no_resched
+#ifdef CONFIG_PREEMPT_COUNT
+#define preempt_enable_no_resched() do { \
+	barrier();           \
+	preempt_count_dec(); \
+	} while (0)
+#else
+#define preempt_enable_no_resched() barrier()
+#endif
+#endif
 
 /* unprivileged user support */
 
 #ifdef STAPCONF_TASK_UID
 #define STP_CURRENT_EUID (current->euid)
 #else
-#ifdef CONFIG_UIDGID_STRICT_TYPE_CHECKS
+#ifdef CONFIG_USER_NS
+#ifndef STAPCONF_FROM_KUID_MUNGED
+#define from_kuid_munged(user_ns, uid) ((uid))
+#define from_kgid_munged(user_ns, gid) ((gid))
+#endif /* !STAPCONF_FROM_KUID_MUNGED */
 #define STP_CURRENT_EUID (from_kuid_munged(current_user_ns(), task_euid(current)))
 #else
 #define STP_CURRENT_EUID (task_euid(current))
