@@ -1,6 +1,6 @@
 /* -*- linux-c -*- 
  * Symbolic Lookup Functions
- * Copyright (C) 2005-2013 Red Hat Inc.
+ * Copyright (C) 2005-2014 Red Hat Inc.
  * Copyright (C) 2006 Intel Corporation.
  *
  * This file is part of systemtap, and is free software.  You can
@@ -15,8 +15,10 @@
 #include "sym.h"
 #include "vma.c"
 #include "stp_string.c"
+#include <asm/unaligned.h>
 #include <asm/uaccess.h>
-
+#include <linux/list.h>
+#include <linux/module.h>
 #ifdef STAPCONF_PROBE_KERNEL
 #include <linux/uaccess.h>
 #endif
@@ -76,8 +78,12 @@ static unsigned long _stp_umodule_relocate(const char *path,
     struct _stp_module *m = _stp_modules[i];
 
     if (strcmp(path, m->path)
-	|| m->num_sections != 1
-	|| strcmp(m->sections[0].name, ".dynamic"))
+        || m->num_sections != 1)
+      continue;
+
+    if (!strcmp(m->sections[0].name, ".absolute"))
+      return offset;
+    if (strcmp(m->sections[0].name, ".dynamic"))
       continue;
 
     if (stap_find_vma_map_info_user(tsk->group_leader, m,
@@ -307,7 +313,7 @@ static int _stp_build_id_check (struct _stp_module *m,
 }
 
 
-/* Validate module/kernel based on build-id (if present)
+/* Validate all-modules + kernel based on build-id (if present).
 *  The completed case is the following combination:
 *	   Debuginfo 		 Module			         Kernel	
 * 			   X				X
@@ -321,6 +327,7 @@ static int _stp_module_check(void)
   struct _stp_module *m = NULL;
   unsigned long notes_addr, base_addr;
   unsigned i,j;
+  int rc = 0;
 
 #ifdef STP_NO_BUILDID_CHECK
   return 0;
@@ -329,9 +336,13 @@ static int _stp_module_check(void)
   for (i = 0; i < _stp_num_modules; i++)
     {
       m = _stp_modules[i];
+
       if (m->build_id_len > 0 && m->notes_sect != 0) {
           dbug_sym(1, "build-id validation [%s]\n", m->name); /* kernel only */
 
+          /* skip userspace program */
+          if (m->name[0] != '/') continue;
+
           /* notes end address */
           if (!strcmp(m->name, "kernel")) {
               notes_addr = _stp_kmodule_relocate("kernel",
@@ -348,10 +359,12 @@ static int _stp_module_check(void)
                   notes_addr, base_addr);
               continue;
           }
-          return _stp_build_id_check (m, notes_addr, NULL);
+
+          rc |=  _stp_build_id_check (m, notes_addr, NULL);
       } /* end checking */
     } /* end loop */
-  return 0;
+
+  return rc;
 }
 
 
@@ -368,9 +381,13 @@ static int _stp_kmodule_check (const char *name)
   return 0;
 #endif
 
+  WARN_ON(!name || name[0]=='/'); // non-userspace only
+
   for (i = 0; i < _stp_num_modules; i++)
     {
       m = _stp_modules[i];
+
+      /* PR16406 must be unique kernel module name (non-/-prefixed path) */
       if (strcmp (name, m->name)) continue;
 
       if (m->build_id_len > 0 && m->notes_sect != 0) {
@@ -389,7 +406,7 @@ static int _stp_kmodule_check (const char *name)
       } /* end checking */
     } /* end loop */
 
-  return 0; /* name not found */
+  return 0; /* not found */
 }
 
 
@@ -407,11 +424,15 @@ static int _stp_usermodule_check(struct task_struct *tsk, const char *path_name,
   return 0;
 #endif
 
+  WARN_ON(!path_name || path_name[0]!='/'); // user-space only
+
   for (i = 0; i < _stp_num_modules; i++)
     {
       m = _stp_modules[i];
-      if (strcmp(path_name, _stp_modules[i]->path) != 0)
-	continue;
+
+      /* PR16406 must be unique userspace name (/-prefixed path); it's also in m->name */
+      if (strcmp(path_name, m->path) != 0) continue;
+
       if (m->build_id_len > 0) {
 	int ret, build_id_len;
 
@@ -428,7 +449,7 @@ static int _stp_usermodule_check(struct task_struct *tsk, const char *path_name,
       }
     }
 
-  return 0;
+  return 0; /* not found */
 }
 
 
@@ -601,7 +622,6 @@ static void _stp_kmodule_update_address(const char* module,
                                         unsigned long address)
 {
   unsigned mi, si;
-        
   for (mi=0; mi<_stp_num_modules; mi++)
     {
       const char *note_sectname = ".note.gnu.build-id";
