@@ -1,5 +1,5 @@
 /* -*- linux-c -*- 
- * Copyright (C) 2005-2008 Red Hat Inc.
+ * Copyright (C) 2005-2014 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -72,8 +72,8 @@ struct _stp_section {
 };
 
 struct _stp_module {
-        const char* name; /* basename, may be duplicate! */
-        const char* path; /* canonical filesystem path used for runtime matching */
+        const char* name; /* module name (kernel) or /canonical/path for userspace*/
+        const char* path; /* canonical filesystem path (kernel .ko or user) */
 	struct _stp_section *sections;
   	unsigned num_sections;
 
@@ -95,10 +95,9 @@ struct _stp_module {
 	int build_id_len;
 };
 
-
 /* Defined by translator-generated stap-symbols.h. */
 static struct _stp_module *_stp_modules [];
-static unsigned _stp_num_modules;
+static const unsigned _stp_num_modules;
 
 /* Used in the unwinder to special case unwinding through kretprobes. */
 /* Initialized through translator (stap-symbols.h) relative to kernel */
@@ -116,4 +115,10 @@ static void _stp_kmodule_update_address(const char* module,
                                         const char* section,
                                         unsigned long offset);
 
+#if defined(STP_USE_DWARF_UNWINDER) && defined(STP_NEED_UNWIND_DATA)
+static struct _stp_module _stp_module_self;
+static struct _stp_section _stp_module_self_sections[];
+static struct _stp_symbol _stp_module_self_symbols_0[];
+static struct _stp_symbol _stp_module_self_symbols_1[];
+#endif /* defined(STP_USE_DWARF_UNWINDER) && defined(STP_NEED_UNWIND_DATA) */
 #endif /* _STP_SYM_H_ */
