@@ -2,7 +2,7 @@
  * transport.c - stp transport functions
  *
  * Copyright (C) IBM Corporation, 2005
- * Copyright (C) Red Hat Inc, 2005-2011
+ * Copyright (C) Red Hat Inc, 2005-2014
  * Copyright (C) Intel Corporation, 2006
  *
  * This file is part of systemtap, and is free software.  You can
@@ -135,16 +135,20 @@ static void _stp_handle_start(struct _stp_msg_start *st)
 
 		_stp_target = st->target;
 		st->res = systemtap_module_init();
-		if (st->res == 0)
+		if (st->res == 0) {
 			_stp_probes_started = 1;
 
-                /* Register the module notifier. */
-                if (!_stp_module_notifier_active) {
-                        int rc = register_module_notifier(& _stp_module_notifier_nb);
-                        if (rc == 0)
-                                _stp_module_notifier_active = 1;
-                        else
-                                _stp_warn ("Cannot register module notifier (%d)\n", rc);
+                        /* Register the module notifier ... */
+                        /* NB: but not if the module_init stuff
+                           failed: something nasty has happened, and
+                           we want no further probing started.  PR16766 */
+                        if (!_stp_module_notifier_active) {
+                                int rc = register_module_notifier(& _stp_module_notifier_nb);
+                                if (rc == 0)
+                                        _stp_module_notifier_active = 1;
+                                else
+                                        _stp_warn ("Cannot register module notifier (%d)\n", rc);
+                        }
                 }
 
 		/* Called from the user context in response to a proc
@@ -292,11 +296,18 @@ static void _stp_ctl_work_callback(unsigned long val)
 {
 	int do_io = 0;
 	unsigned long flags;
+	struct context* __restrict__ c = NULL;
+
+	/* Prevent probe reentrancy while grabbing probe-used locks.  */
+	c = _stp_runtime_entryfn_get_context();
 
 	spin_lock_irqsave(&_stp_ctl_ready_lock, flags);
 	if (!list_empty(&_stp_ctl_ready_q))
 		do_io = 1;
 	spin_unlock_irqrestore(&_stp_ctl_ready_lock, flags);
+
+	_stp_runtime_entryfn_put_context(c);
+
 	if (do_io)
 		wake_up_interruptible(&_stp_ctl_wq);
 
@@ -337,7 +348,7 @@ static int _stp_transport_init(void)
 	_stp_uid = current->uid;
 	_stp_gid = current->gid;
 #else
-#ifdef CONFIG_UIDGID_STRICT_TYPE_CHECKS
+#if defined(CONFIG_USER_NS) || (LINUX_VERSION_CODE >= KERNEL_VERSION(3,14,0))
 	_stp_uid = from_kuid_munged(current_user_ns(), current_uid());
 	_stp_gid = from_kgid_munged(current_user_ns(), current_gid());
 #else
@@ -416,6 +427,10 @@ static int _stp_transport_init(void)
 	if (_stp_print_init() < 0)
 		goto err2;
 
+	/* set _stp_module_self dynamic info */
+	if (_stp_module_update_self() < 0)
+		goto err3;
+
 	/* start transport */
 	_stp_transport_data_fs_start();
 
