@@ -7,7 +7,7 @@
  * Public License (GPL); either version 2, or (at your option) any
  * later version.
  *
- * Copyright (C) 2007-2013 Red Hat Inc.
+ * Copyright (C) 2007-2014 Red Hat Inc.
  */
 
 #include "staprun.h"
@@ -66,22 +66,6 @@ static char *get_abspath(char *path)
 	return path_buf;
 }
 
-int stap_strfloctime(char *buf, size_t max, const char *fmt, time_t t)
-{
-	struct tm tm;
-	size_t ret;
-	if (buf == NULL || fmt == NULL || max <= 1)
-		return -EINVAL;
-	localtime_r(&t, &tm);
-        /* NB: this following invocation is the reason for staprun's being built
-           with -Wno-format-nonliteral.  strftime parsing does not have security
-           implications AFAIK, but gcc still wants to check them.  */
-	ret = strftime(buf, max, fmt, &tm);
-	if (ret == 0)
-		return -EINVAL;
-	return (int)ret;
-}
-
 int make_outfile_name(char *buf, int max, int fnum, int cpu, time_t t, int bulk)
 {
 	int len;
@@ -222,7 +206,7 @@ void parse_args(int argc, char **argv)
 			break;
 		case 'V':
                         eprintf(_("Systemtap module loader/runner (version %s, %s)\n"
-                              "Copyright (C) 2005-2013 Red Hat, Inc. and others\n"
+                              "Copyright (C) 2005-2014 Red Hat, Inc. and others\n"
                               "This is free software; see the source for copying conditions.\n"),
                             VERSION, STAP_EXTENDED_VERSION);
                         _exit(1);
@@ -636,8 +620,10 @@ char *parse_stap_color(const char *type)
 	int done = 0;
 
 	key = getenv("SYSTEMTAP_COLORS");
-	if (key == NULL || *key == '\0')
+	if (key == NULL)
 		key = "error=01;31:warning=00;33:source=00;34:caret=01:token=01";
+	else if (*key == '\0')
+		return NULL; // disable colors if set but empty
 
 	while (!done) {
 		if (!(col = strchr(key, ':'))) {
