@@ -1,5 +1,5 @@
 // tapset for kernel static markers
-// Copyright (C) 2005-2010 Red Hat Inc.
+// Copyright (C) 2005-2014 Red Hat Inc.
 // Copyright (C) 2005-2007 Intel Corporation.
 //
 // This file is part of systemtap, and is free software.  You can
@@ -140,9 +140,7 @@ mark_var_expanding_visitor::visit_target_symbol_context (target_symbol* e)
  else if (e->name == "$$vars" || e->name == "$$parms")
   {
      //copy from tracepoint
-     token* pf_tok = new token(*e->tok);
-     pf_tok->content = "sprintf";
-     print_format* pf = print_format::create(pf_tok);
+     print_format* pf = print_format::create(e->tok, "sprintf");
 
      for (unsigned i = 0; i < mark_args.size(); ++i)
         {
@@ -571,6 +569,9 @@ private:
     mark_cache_const_iterator_pair_t;
   mark_cache_t mark_cache;
 
+  string suggest_marks(systemtap_session& sess,
+                       const string& mark);
+
 public:
   mark_builder(): cache_initialized(false) {}
 
@@ -592,6 +593,30 @@ public:
 };
 
 
+string
+mark_builder::suggest_marks(systemtap_session& sess,
+                           const string& mark)
+{
+  if (mark.empty() || mark_cache.empty())
+    return "";
+
+  set<string> marks;
+
+  // Collect all markers from cache
+  for (mark_cache_const_iterator_t it = mark_cache.begin();
+       it != mark_cache.end(); it++)
+    marks.insert(it->first);
+
+  if (sess.verbose > 2)
+    clog << "suggesting from " << marks.size()
+         << " kernel marks" << endl;
+
+  if (marks.empty())
+    return "";
+
+  return levenshtein_suggest(mark, marks, 5); // print top 5 marks only
+}
+
 void
 mark_builder::build(systemtap_session & sess,
 		    probe * base,
@@ -671,6 +696,8 @@ mark_builder::build(systemtap_session & sess,
       module_markers.close();
     }
 
+  unsigned results_pre = finished_results.size();
+
   // Search marker list for matching markers
   for (mark_cache_const_iterator_t it = mark_cache.begin();
        it != mark_cache.end(); it++)
@@ -696,6 +723,16 @@ mark_builder::build(systemtap_session & sess,
 	    }
 	}
     }
+
+  if (results_pre == finished_results.size() && !loc->from_glob)
+    {
+      string sugs = suggest_marks(sess, mark_str_val);
+      if (!sugs.empty())
+        throw SEMANTIC_ERROR (_NF("no match (similar mark: %s)",
+                                  "no match (similar marks: %s)",
+                                  sugs.find(',') == string::npos,
+                                  sugs.c_str()));
+    }
 }
 
 
